@@ -29,7 +29,7 @@ require_once KRONOLITH_BASE . '/lib/Recurrence.php';
  *
  * Act on alarms in events and send emails/pages/etc. to users.
  *
- * $Horde: kronolith/lib/Scheduler/kronolith.php,v 1.25.6.21 2008/09/02 21:59:37 jan Exp $
+ * $Horde: kronolith/lib/Scheduler/kronolith.php,v 1.25.6.22 2008/10/14 17:17:09 jan Exp $
  *
  * @package Horde_Scheduler
  */
@@ -129,21 +129,79 @@ class Horde_Scheduler_kronolith extends Horde_Scheduler {
             return;
         }
 
-        $alarms = Kronolith::listAlarms(new Horde_Date($this->_runtime), array_keys($this->_calendars));
-        foreach ($alarms as $calId => $calarms) {
-            $GLOBALS['kronolith_driver']->open($calId);
-            foreach ($calarms as $eventId) {
-                $event = $GLOBALS['kronolith_driver']->getEvent($eventId);
-                if (is_a($event, 'PEAR_Error')) {
+        require_once 'Horde/Group.php';
+        require_once 'Horde/Identity.php';
+        require_once 'Horde/MIME.php';
+        require_once 'Horde/MIME/Headers.php';
+        require_once 'Horde/MIME/Message.php';
+
+        // Retrieve a list of users associated with each calendar, and
+        // thus a list of users who have used kronolith and
+        // potentially have a calendar with alarms set.
+        $users = array();
+        $groupManager = &Group::singleton();
+
+        foreach (array_keys($this->_calendars) as $calendarId) {
+            $calendar = $GLOBALS['shares']->getShare($calendarId);
+            if (is_a($calendar, 'PEAR_Error')) {
+                continue;
+            }
+            $users = array_merge($users, $calendar->listUsers(PERMS_READ));
+            $groups = $calendar->listGroups(PERMS_READ);
+            foreach ($groups as $gid) {
+                $group = $groupManager->getGroupById($gid);
+                $users = array_merge($users, $group->listAllUsers());
+            }
+        }
+
+        // Remove duplicates.
+        $users = array_unique($users);
+
+        // Loop through the users and generate reminders
+        foreach ($users as $user) {
+            $prefs = &Prefs::singleton($GLOBALS['conf']['prefs']['driver'],
+                                       'kronolith', $user);
+            $prefs->retrieve();
+
+            // Set the timezone.
+            @putenv('TZ=' . $prefs->getValue('timezone'));
+
+            $alarms = Kronolith::listAlarms(new Horde_Date($this->_runtime),
+                                            array_keys($this->_calendars));
+            foreach ($alarms as $calId => $calarms) {
+                $GLOBALS['kronolith_driver']->open($calId);
+                $calendar = $GLOBALS['shares']->getShare($calId);
+                $perms = $GLOBALS['shares']->getPermissions($calendar, $user);
+                if (!isset($perms) || ($perms & PERMS_READ) == 0) {
                     continue;
                 }
+                foreach ($calarms as $eventId) {
+                    $event = $GLOBALS['kronolith_driver']->getEvent($eventId);
+                    if (is_a($event, 'PEAR_Error')) {
+                        continue;
+                    }
 
-                $seenid = $eventId . $event->start->timestamp() . $event->getAlarm();
-                if (!isset($this->_seen[$seenid])) {
-                    $this->_seen[$seenid] = $event->start->timestamp() + ($event->durMin * 60);
-                    $result = $this->remind($calId, $event);
-                    if (is_a($result, 'PEAR_Error')) {
-                        Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+                    if ($event->recurs()) {
+                        /* Set the event's start date to the next recurrence
+                         * date. This should avoid problems when an alarm
+                         * triggers on a different day from the actual event,
+                         * and make $seenid unique for each occurrence of a
+                         * recurring event. */
+                        $event->start = $event->recurrence->nextRecurrence($this->_runtime);
+
+                        /* Check for exceptions; do nothing if one is found. */
+                        if ($event->recurrence->hasException($event->start->year, $event->start->month, $event->start->mday)) {
+                            continue;
+                        }
+                    }
+
+                    $seenid = $eventId . $user . $event->start->timestamp() . $event->getAlarm();
+                    if (!isset($this->_seen[$seenid])) {
+                        $this->_seen[$seenid] = $event->start->timestamp() + ($event->durMin * 60);
+                        $result = $this->remind($calId, $event, $user);
+                        if (is_a($result, 'PEAR_Error')) {
+                            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+                        }
                     }
                 }
             }
@@ -151,7 +209,8 @@ class Horde_Scheduler_kronolith extends Horde_Scheduler {
 
         // Discard seen ids that are now in the past (garbage collection).
         foreach (array_keys($this->_seen) as $seenid) {
-            if ($this->_runtime > $this->_seen[$seenid] || $this->_seen[$seenid] === true) {
+            if ($this->_runtime > $this->_seen[$seenid] ||
+                $this->_seen[$seenid] === true) {
                 unset($this->_seen[$seenid]);
             }
         }
@@ -159,18 +218,12 @@ class Horde_Scheduler_kronolith extends Horde_Scheduler {
 
     /**
      */
-    function remind($calId, $event)
+    function remind($calId, $event, $user)
     {
         if ($GLOBALS['kronolith_driver']->getCalendar() != $calId) {
             $GLOBALS['kronolith_driver']->open($calId);
         }
 
-        require_once 'Horde/Group.php';
-        require_once 'Horde/Identity.php';
-        require_once 'Horde/MIME.php';
-        require_once 'Horde/MIME/Headers.php';
-        require_once 'Horde/MIME/Message.php';
-
         /* Desired logic: list users and groups that can view $calId, and send
          * email to any of them that we can find an email address for. This
          * will hopefully be improved at some point so that people don't get
@@ -181,68 +234,34 @@ class Horde_Scheduler_kronolith extends Horde_Scheduler {
             return $share;
         }
 
-        $recipients = array();
-        $emails = array();
-
-        $users = $share->listUsers(PERMS_READ);
-        foreach ($users as $user) {
-            if (empty($emails[$user])) {
-                $identity = &Identity::singleton('none', $user);
-                $email = $identity->getValue('from_addr');
-                if (strstr($email, '@')) {
-                    list($mailbox, $host) = explode('@', $email);
-                    $emails[$user] = MIME::rfc822WriteAddress($mailbox, $host, $identity->getValue('fullname'));
-                }
-            }
-
-            if (!empty($emails[$user])) {
-                $prefs = &Prefs::singleton($GLOBALS['conf']['prefs']['driver'],
-                                           'kronolith', $user);
-                $prefs->retrieve();
-                $shown_calendars = unserialize($prefs->getValue('display_cals'));
-                $reminder = $prefs->getValue('event_reminder');
-                if (($reminder == 'owner' && $user == $share->get('owner')) ||
-                    ($reminder == 'show' && in_array($calId, $shown_calendars)) ||
-                    $reminder == 'read') {
-                    $lang = $prefs->getValue('language');
-                    $twentyFour = $prefs->getValue('twentyFour');
-                    $dateFormat = $prefs->getValue('date_format');
-                    if (!isset($recipients[$lang][$twentyFour][$dateFormat])) {
-                        $recipients[$lang][$twentyFour][$dateFormat] = array();
-                    }
-                    $recipients[$lang][$twentyFour][$dateFormat][] = $emails[$user];
-                }
-            }
-        }
-
-        $groups = $share->listGroups(PERMS_READ);
-        $groupManager = &Group::singleton();
-        foreach ($groups as $gid) {
-            if (empty($emails[$gid])) {
-                $group = $groupManager->getGroupById($gid);
-                if ($email = $group->get('email')) {
-                    $emails[$gid] = $group->get('email');
-                }
-            }
-
-            if (!empty($emails[$gid])) {
-                $prefs = &Prefs::singleton($GLOBALS['conf']['prefs']['driver'], 'horde', $gid);
-                $prefs->retrieve();
-                $lang = $prefs->getValue('language');
-                $twentyFour = $prefs->getValue('twentyFour');
-                $dateFormat = $prefs->getValue('date_format');
-                if (!isset($recipients[$lang][$twentyFour][$dateFormat])) {
-                    $recipients[$lang][$twentyFour][$dateFormat] = array();
-                }
-                $recipients[$lang][$twentyFour][$dateFormat][] = $emails[$gid];
-            }
-        }
-
-        if (!$recipients) {
-            Horde::logMessage(sprintf('No email addresses available to send reminder for %s to recipient(s): %s %s', $event->title, implode(', ', $users), implode(', ', $groups)), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        if ($event->isPrivate() && $event->getCreatorId() != $user) {
             return false;
         }
 
+        $prefs = &Prefs::singleton($GLOBALS['conf']['prefs']['driver'],
+                                   'kronolith', $user);
+        $prefs->retrieve();
+        $identity = &Identity::singleton('none', $user);
+        $email = $identity->getValue('from_addr');
+        if (strstr($email, '@')) {
+            list($mailbox, $host) = explode('@', $email);
+            $email = MIME::rfc822WriteAddress($mailbox, $host, $identity->getValue('fullname'));
+        }
+
+        $shown_calendars = unserialize($prefs->getValue('display_cals'));
+        $reminder = $prefs->getValue('event_reminder');
+        if (($reminder == 'owner' && $user == $share->get('owner')) ||
+            ($reminder == 'show' && in_array($calId, $shown_calendars)) ||
+            $reminder == 'read') {
+            // Noop.
+        } else {
+            return false;
+        }
+
+        $lang = $prefs->getValue('language');
+        $twentyFour = $prefs->getValue('twentyFour');
+        $dateFormat = $prefs->getValue('date_format');
+
         $msg_headers = new MIME_Headers();
         $msg_headers->addMessageIdHeader();
         $msg_headers->addAgentHeader();
@@ -258,36 +277,30 @@ class Horde_Scheduler_kronolith extends Horde_Scheduler {
             return;
         }
 
-        foreach ($recipients as $lang => $twentyFour) {
-            NLS::setLang($lang);
-            NLS::setTextdomain('kronolith', KRONOLITH_BASE . '/locale', NLS::getCharset());
-            String::setDefaultCharset(NLS::getCharset());
+        NLS::setLang($lang);
+        NLS::setTextdomain('kronolith', KRONOLITH_BASE . '/locale', NLS::getCharset());
+        String::setDefaultCharset(NLS::getCharset());
 
-            $msg_headers->removeHeader('Subject');
-            $msg_headers->addHeader('Subject', sprintf(_("Reminder: %s"), $event->title));
+        $msg_headers->removeHeader('Subject');
+        $msg_headers->addHeader('Subject', sprintf(_("Reminder: %s"), $event->title));
 
-            foreach ($twentyFour as $tf => $dateFormat) {
-                foreach ($dateFormat as $df => $df_recipients) {
-                    $message = "\n" . sprintf(_("We would like to remind you of this upcoming event.\n\n%s\n\nLocation: %s\n\nDate: %s\nTime: %s\n\n%s"),
-                                              $event->title,
-                                              $event->location,
-                                              strftime($df, $event->start->timestamp()),
-                                              date($tf ? 'H:i' : 'h:ia', $event->start->timestamp()),
-                                              $event->getDescription());
+        $message = "\n" . sprintf(_("We would like to remind you of this upcoming event.\n\n%s\n\nLocation: %s\n\nDate: %s\nTime: %s\n\n%s"),
+                                  $event->title,
+                                  $event->location,
+                                  strftime($dateFormat, $event->start->timestamp()),
+                                  date($twentyFour ? 'H:i' : 'h:ia', $event->start->timestamp()),
+                                  $event->getDescription());
 
-                    $mime = new MIME_Message();
-                    $body = new MIME_Part('text/plain', String::wrap($message, 76, "\n"), NLS::getCharset());
+        $mime = new MIME_Message();
+        $body = new MIME_Part('text/plain', String::wrap($message, 76, "\n"), NLS::getCharset());
 
-                    $mime->addPart($body);
-                    $msg_headers->addMIMEHeaders($mime);
+        $mime->addPart($body);
+        $msg_headers->addMIMEHeaders($mime);
 
-                    Horde::logMessage(sprintf('Sending reminder for %s to %s', $event->title, implode(', ', $df_recipients)), __FILE__, __LINE__, PEAR_LOG_DEBUG);
-                    $sent = $mime->send(implode(', ', $df_recipients), $msg_headers, $mail_driver, $mail_params);
-                    if (is_a($sent, 'PEAR_Error')) {
-                        return $sent;
-                    }
-                }
-            }
+        Horde::logMessage(sprintf('Sending reminder for %s to %s', $event->title, $email), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        $sent = $mime->send($email, $msg_headers, $mail_driver, $mail_params);
+        if (is_a($sent, 'PEAR_Error')) {
+            return $sent;
         }
 
         return true;
